@@ -31,15 +31,12 @@ from __future__ import absolute_import
 import platform
 from threading import Thread
 
-from time import mktime
 from datetime import datetime, timedelta
 
 import io
 import json
 import os
 import itertools
-import codecs
-
 
 
 try:
@@ -56,8 +53,8 @@ import dateutil.parser
 from . import __title__, __version__
 from .compat import (urlsplit, queryencode,
                      range, basestring)
-from .error import (KnownResponseError, ResponseError, NoSuchBucket, AccessDenied,
-                    InvalidArgumentError, InvalidSizeError, InvalidXMLError, NoSuchBucketPolicy)
+from .error import (ResponseError, NoSuchBucket, AccessDenied,
+                    InvalidArgumentError, InvalidSizeError, InvalidXMLError)
 from .definitions import Object, UploadPart
 from .parsers import (parse_list_buckets,
                       parse_list_objects,
@@ -72,18 +69,20 @@ from .parsers import (parse_list_buckets,
                       parse_multi_object_delete_response)
 from .helpers import (get_target_url, is_non_empty_string,
                       is_valid_endpoint,
-                      get_sha256_hexdigest, get_md5_base64digest, Hasher,
+                      get_sha256_hexdigest, get_md5_base64digest,
                       optimal_part_info,
-                      is_valid_bucket_name, PartMetadata, read_full,
+                      is_valid_bucket_name, read_full,
                       get_s3_region_from_endpoint, is_valid_sse_object, is_valid_sse_c_object,
                       is_valid_source_sse_object,
                       is_valid_bucket_notification_config, is_valid_policy_type,
-                      get_s3_region_from_endpoint,
                       mkdir_p, dump_http, amzprefix_user_metadata,
                       is_supported_header,is_amz_header)
 from .helpers import (MAX_MULTIPART_OBJECT_SIZE,
+                      MAX_PART_SIZE,
                       MAX_POOL_SIZE,
-                      MIN_PART_SIZE)
+                      MIN_PART_SIZE,
+                      DEFAULT_PART_SIZE,
+                      MAX_MULTIPART_COUNT)
 from .signer import (sign_v4, presign_v4,
                      generate_credential_string,
                      post_presign_signature)
@@ -130,6 +129,7 @@ class Minio(object):
     :param endpoint: Hostname of the cloud storage server.
     :param access_key: Access key to sign self._http.request with.
     :param secret_key: Secret key to sign self._http.request with.
+    :param session_token: Session token to sign self._http.request with.
     :param secure: Set this value if wish to make secure requests.
          Default is True.
     :param region: Set this value to override automatic bucket
@@ -139,7 +139,9 @@ class Minio(object):
     :return: :class:`Minio <Minio>` object
     """
     def __init__(self, endpoint, access_key=None,
-                 secret_key=None, secure=True,
+                 secret_key=None,
+                 session_token=None,
+                 secure=True,
                  region=None,
                  http_client=None):
 
@@ -169,6 +171,7 @@ class Minio(object):
         self._is_ssl = secure
         self._access_key = access_key
         self._secret_key = secret_key
+        self._session_token = session_token
         self._user_agent = _DEFAULT_USER_AGENT
         self._trace_output_stream = None
 
@@ -287,7 +290,9 @@ class Minio(object):
         # Get signature headers if any.
         headers = sign_v4(method, url, region,
                           headers, self._access_key,
-                          self._secret_key, content_sha256_hex)
+                          self._secret_key,
+                          self._session_token,
+                          content_sha256_hex)
 
         response = self._http.urlopen(method, url,
                                       body=content,
@@ -324,7 +329,9 @@ class Minio(object):
         # Get signature headers if any.
         headers = sign_v4(method, url, region,
                           headers, self._access_key,
-                          self._secret_key, None)
+                          self._secret_key,
+                          self._session_token,
+                          None)
 
         response = self._http.urlopen(method, url,
                                       body=None,
@@ -354,9 +361,9 @@ class Minio(object):
         try:
             self._url_open('HEAD', bucket_name=bucket_name)
         # If the bucket has not been created yet, Minio will return a "NoSuchBucket" error.
-        except NoSuchBucket as e:
+        except NoSuchBucket:
             return False
-        except ResponseError as e:
+        except ResponseError:
             raise
         return True
 
@@ -537,7 +544,8 @@ class Minio(object):
 
     def fput_object(self, bucket_name, object_name, file_path,
                     content_type='application/octet-stream',
-                    metadata=None, sse=None, progress=None):
+                    metadata=None, sse=None, progress=None,
+                    part_size=DEFAULT_PART_SIZE):
         """
         Add a new object to the cloud storage server.
 
@@ -551,6 +559,7 @@ class Minio(object):
         :param metadata: Any additional metadata to be uploaded along
             with your PUT request.
         :param progress: A progress object
+        :param part_size: Multipart part size
         :return: etag
         """
 
@@ -558,7 +567,8 @@ class Minio(object):
         with open(file_path, 'rb') as file_data:
             file_size = os.stat(file_path).st_size
             return self.put_object(bucket_name, object_name, file_data,
-                                   file_size, content_type, metadata, sse, progress)
+                                   file_size, content_type, metadata, sse,
+                                   progress, part_size)
 
     def fget_object(self, bucket_name, object_name, file_path, request_headers=None, sse=None):
         """
@@ -747,7 +757,8 @@ class Minio(object):
 
     def put_object(self, bucket_name, object_name, data, length,
                    content_type='application/octet-stream',
-                   metadata=None, sse=None, progress=None):
+                   metadata=None, sse=None, progress=None,
+                   part_size=DEFAULT_PART_SIZE):
         """
         Add a new object to the cloud storage server.
 
@@ -771,6 +782,7 @@ class Minio(object):
         :param metadata: Any additional metadata to be uploaded along
             with your PUT request.
         :param progress: A progress object
+        :param part_size: Multipart part size
         :return: etag
         """
 
@@ -788,9 +800,18 @@ class Minio(object):
             raise ValueError(
                 'Invalid input data does not implement a callable read() method')
 
-        if length > MAX_MULTIPART_OBJECT_SIZE:
-            raise InvalidArgumentError('Input content size is bigger '
-                                       ' than allowed maximum of 5TiB.')
+        if length > (part_size * MAX_MULTIPART_COUNT):
+            raise InvalidArgumentError('Part size * max_parts(10000) is '
+                                       ' lesser than input length.')
+
+        if part_size < MIN_PART_SIZE:
+            raise InvalidArgumentError('Input part size is smaller '
+                                       ' than allowed minimum of 5MiB.')
+
+        if part_size > MAX_PART_SIZE:
+            raise InvalidArgumentError('Input part size is bigger '
+                                       ' than allowed maximum of 5GiB.')
+
         if not metadata:
             metadata = {}
 
@@ -799,10 +820,11 @@ class Minio(object):
         metadata['Content-Type'] = 'application/octet-stream' if \
             not content_type else content_type
 
-        if length > MIN_PART_SIZE:
+        if length > part_size:
             return self._stream_put_object(bucket_name, object_name,
                                            data, length, metadata=metadata,
-                                           sse=sse, progress=progress)
+                                           sse=sse, progress=progress,
+                                           part_size=part_size)
 
         current_data = data.read(length)
         if len(current_data) != length:
@@ -1293,6 +1315,7 @@ class Minio(object):
         return presign_v4(method, url,
                           self._access_key,
                           self._secret_key,
+                          session_token=self._session_token,
                           region=region,
                           expires=int(expires.total_seconds()),
                           response_headers=response_headers,
@@ -1384,21 +1407,29 @@ class Minio(object):
         credential_string = generate_credential_string(self._access_key,
                                                        date, region)
 
-        post_policy_base64 = post_policy.base64(extras=[
+        policy = [
             ('eq', '$x-amz-date', iso8601_date),
             ('eq', '$x-amz-algorithm', _SIGN_V4_ALGORITHM),
             ('eq', '$x-amz-credential', credential_string),
-        ])
+        ]
+        if self._session_token:
+            policy.add(('eq', '$x-amz-security-token', self._session_token))
+
+        post_policy_base64 = post_policy.base64(extras=policy)
         signature = post_presign_signature(date, region,
                                            self._secret_key,
                                            post_policy_base64)
-        post_policy.form_data.update({
+        form_data = {
             'policy': post_policy_base64,
             'x-amz-algorithm': _SIGN_V4_ALGORITHM,
             'x-amz-credential': credential_string,
             'x-amz-date': iso8601_date,
             'x-amz-signature': signature,
-        })
+        }
+        if self._session_token:
+            form_data['x-amz-security-token'] = self._session_token
+
+        post_policy.form_data.update(form_data)
         url_str = get_target_url(self._endpoint_url,
                                  bucket_name=post_policy.form_data['bucket'],
                                  bucket_region=region)
@@ -1537,7 +1568,8 @@ class Minio(object):
 
     def _stream_put_object(self, bucket_name, object_name,
                            data, content_size,
-                           metadata=None, sse=None, progress=None):
+                           metadata=None, sse=None,
+                           progress=None, part_size=MIN_PART_SIZE):
         """
         Streaming multipart upload operation.
 
@@ -1549,6 +1581,7 @@ class Minio(object):
         :param metadata: Any additional metadata to be uploaded along
            with your object.
         :param progress: A progress object
+        :param part_size: Multipart part size
         """
         is_valid_bucket_name(bucket_name)
         is_non_empty_string(object_name)
@@ -1566,7 +1599,7 @@ class Minio(object):
 
         # Calculate optimal part info.
         total_parts_count, part_size, last_part_size = optimal_part_info(
-            content_size)
+            content_size, part_size)
 
         # Instantiate a thread pool with 3 worker threads
         pool = ThreadPool(_PARALLEL_UPLOADERS)
@@ -1779,7 +1812,9 @@ class Minio(object):
         # Get signature headers if any.
         headers = sign_v4(method, url, region,
                           headers, self._access_key,
-                          self._secret_key, None)
+                          self._secret_key,
+                          self._session_token,
+                          None)
 
         response = self._http.urlopen(method, url,
                                       body=None,
@@ -1828,7 +1863,8 @@ class Minio(object):
         # Get signature headers if any.
         headers = sign_v4(method, url, region,
                           fold_case_headers, self._access_key,
-                          self._secret_key, content_sha256)
+                          self._secret_key, self._session_token,
+                          content_sha256)
 
         response = self._http.urlopen(method, url,
                                       body=body,
